'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapObject = exports.entityMapToIndexes = exports.flatten = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _objectUtils = require('./utils/object-utils');

var objectUtils = _interopRequireWildcard(_objectUtils);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var flatten = exports.flatten = function flatten(array) {
  return array.reduce(function (flattenedArray, item) {
    var flattenedItem = Array.isArray(item) ? flatten(item) : item;
    return flattenedArray.concat(flattenedItem);
  }, []);
};

var entityMapToIndexes = exports.entityMapToIndexes = function entityMapToIndexes(entityMap, tables, indexKeys, primaryKey) {
  return tables.reduce(function (indexMap, tableName) {
    entityMap[tableName].forEach(function (_ref) {
      var path = _ref.path;
      var entity = _ref.entity;

      indexKeys.forEach(function (currentIndexKey) {
        indexMap = addEntityToIndexMap(entity, path, tableName, primaryKey, currentIndexKey, indexMap);
      });
    });

    return indexMap;
  }, {});
};

var addEntityToIndexMap = function addEntityToIndexMap(entity, pathInTree, tableName, primaryKey, currentIndexKey, indexMap) {
  var path = [tableName, currentIndexKey, entity[currentIndexKey]];

  if (currentIndexKey === primaryKey && !entity[primaryKey]) {
    throw new Error('\n      Cannot create indexes: ->\n        Primary key ' + primaryKey + ' of entity ' + entity + ' has a falsy value: ' + entity[primaryKey] + '!\n    ');
  }

  objectUtils.ensurePathInObject(indexMap, path, []);

  var indexValues = objectUtils.getInPath(indexMap, path);

  if (currentIndexKey === primaryKey && indexValues.length === 1) {
    throw new Error('\n      Cannot create indexes: ->\n        Duplicate primary key ' + entity[primaryKey] + ' in table \'' + tableName + '\'\n    ');
  }

  indexValues.push(pathInTree);

  return indexMap;
};

var mapObject = exports.mapObject = function mapObject(object, tables, indexKeys, primaryKey) {
  return tables.reduce(function (indexes, tableName) {
    return object[tableName].reduce(function (indexMap, entity, entityIndex) {
      indexKeys.forEach(function (key) {
        indexMap = addEntityToIndexMap(entity, [tableName, entityIndex], tableName, primaryKey, key, indexMap);
      }, indexMap);
      return indexMap;
    }, indexes);
  }, {});
};

var TreeIndexedMapper = function () {
  function TreeIndexedMapper(driver, indexedMap, indexes, primaryKey) {
    _classCallCheck(this, TreeIndexedMapper);

    this._driver = driver;
    this._indexedMap = indexedMap;
    this._indexes = indexes;
    this._primaryKey = primaryKey;
  }

  _createClass(TreeIndexedMapper, [{
    key: '_normalizeSelector',
    value: function _normalizeSelector(maybeSelector) {
      if (!objectUtils.isObject(maybeSelector)) {
        var objectSelector = {};
        objectSelector[this._primaryKey] = maybeSelector;
        return objectSelector;
      }

      return maybeSelector;
    }
  }, {
    key: '_selectorToPath',
    value: function _selectorToPath(selector) {
      var arrayOfPaths = Object.keys(selector).map(function (selectorKey) {
        return [selectorKey, selector[selectorKey]];
      });

      return flatten(arrayOfPaths);
    }
  }, {
    key: '_getBySelector',
    value: function _getBySelector(tableName, selector) {
      var _this = this;

      var withPaths = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];

      if (this._indexedMap[tableName] === undefined) {
        throw new Error('\n        Cannot get entity: ->\n          Trying to query a table "' + tableName + '" that does not exist.\n      ');
      }

      var matchedPaths = this.getPathsBySelector(tableName, selector);

      return matchedPaths.map(function (path) {
        var value = _this._driver.getInPath(path);
        return withPaths ? { path: path, value: value } : value;
      });
    }
  }, {
    key: 'getPathsBySelector',
    value: function getPathsBySelector(tableName, selector) {
      var _this2 = this;

      return Object.keys(selector).reduce(function (result, selectorKey) {
        var indexTable = objectUtils.getInPath(_this2._indexedMap, [tableName, selectorKey]);
        if (indexTable === undefined) {
          throw new Error('Cannot get entity: ->\n            Database table ' + tableName + ' was not indexed using key "' + selectorKey + '".\n            Try different selector with these keys: "' + _this2._indexes.join(', ') + '"');
        }

        try {
          var pathToEntites = objectUtils.getInPath(_this2._indexedMap, [tableName, selectorKey, selector[selectorKey]]) || [];
          return result.concat(pathToEntites);
        } catch (err) {
          return result;
        }
      }, []);
    }
  }, {
    key: 'updateInPath',
    value: function updateInPath(path, value) {
      return this._driver.updateInPath(path, value);
    }
  }, {
    key: 'get',
    value: function get(tableName) {
      var primaryKeyValue = arguments.length <= 1 || arguments[1] === undefined ? undefined : arguments[1];

      if (primaryKeyValue === undefined) {
        return this._driver.getInPath([tableName]);
      }

      var selector = [];
      selector[this._primaryKey] = primaryKeyValue;
      var result = this._getBySelector(tableName, selector);
      return result[0] || null;
    }
  }, {
    key: 'getBy',
    value: function getBy(tableName, selector) {
      var first = arguments.length <= 2 || arguments[2] === undefined ? false : arguments[2];

      selector = this._normalizeSelector(selector);

      var result = this._getBySelector(tableName, selector);
      return first ? result[0] || null : this._driver.asArray(result);
    }
  }, {
    key: 'update',
    value: function update(tableName, selector, updater) {
      var _this3 = this;

      selector = this._normalizeSelector(selector);

      this._getBySelector(tableName, selector, true).forEach(function (object) {
        var newValue = typeof updater === 'function' ? updater(object.value) : updater;

        if (object.value[_this3._primaryKey] !== newValue[_this3._primaryKey]) {
          throw new Error('\n          Cannot update an entity: ->\n            You are trying to update primary key "' + _this3._primaryKey + '"\n            from "' + object.value[_this3._primaryKey] + '" to "' + newValue[_this3._primaryKey] + '" which is not allowed!\n        ');
        }

        _this3.updateInPath(object.path, newValue);
      });

      return this;
    }
  }, {
    key: 'delete',
    value: function _delete(tableName, selector) {
      var _this4 = this;

      selector = this._normalizeSelector(selector);

      var matchedPaths = this.getPathsBySelector(tableName, selector);
      matchedPaths.forEach(function (path) {
        return _this4._driver.deleteInPath(path);
      });

      var distinctRoots = matchedPaths.reduce(function (roots, path) {
        var root = path[0];
        if (!roots[root] && root) {
          roots[root] = root;
        }
        return roots;
      }, {});

      Object.keys(distinctRoots).forEach(function (root) {
        _this4._indexedMap = _this4._remapEntities(root, _this4._driver.getInPath([root]));
      });

      return this;
    }
  }, {
    key: '_remapEntities',
    value: function _remapEntities(tableName, entities) {
      var newTree = {};
      newTree[tableName] = entities;
      var indexes = mapObject(newTree, Object.keys(newTree), this._indexes, this._primaryKey);
      return _extends({}, this._indexedMap, indexes);
    }
  }, {
    key: 'add',
    value: function add(tableName, entity) {
      var primaryKeyValue = entity[this._primaryKey];

      if (primaryKeyValue === undefined) {
        throw new Error('\n        Cannot add an entity: ->\n          Entity does not contain a primary key: "' + this._primaryKey + '"!\n      ');
      }

      var existingEntity = this.get(tableName, primaryKeyValue);

      if (existingEntity) {
        throw new Error('\n        Cannot add an entity: ->\n          Duplicated value of primary key "' + this._primaryKey + ':' + primaryKeyValue + '" in existing entity ' + existingEntity + '!\n      ');
      }

      var entities = (this._driver.getInPath([tableName]) || []).concat([entity]);
      this._indexedMap = this._remapEntities(tableName, entities);
      this._driver.addInPath([tableName], entity);

      return this;
    }
  }, {
    key: 'getTree',
    value: function getTree() {
      return this._driver.getTree();
    }
  }]);

  return TreeIndexedMapper;
}();

exports.default = TreeIndexedMapper;